/* BATTLE OVERLAY README:
BattleOverlay is a little helper to draw on top of all battlers.
Sometimes drawing in the onBattlerDrawn-callback has the disadvantage that bigger monster sprites may hide UI elements drawn on top of other monsters.

To draw that kind of UI elements like lifebars and condition icons, we would want to draw them on top of all battlers,
but in an ordner that matches the y-coordinate of the battler-sprites.

This component does that by saving the IDs of drawn actors into an array and executing a custom callback cycle after all actors are drawn.
The array method was chosen in favor of manually checking the party and monsters-arrays and sorting them by y-coordinate.
The rpg maker actually did that for us as part of the original drawing order.

    HOW TO USE:
- Function 'RememberBattlerDrawn' needs to be called in dyn-callback 'onBattlerDrawn'
- Function 'DrawOverlay' needs to be called in dyn-calback 'onDrawBattleActionWindow' with its 'selActive' parameter being false.
- Implement your own code in the OnOverlayDrawActor() and OnOverlayDrawMonster() functions.

Author: Corti, 2015
*/

#ifndef BATTLEOVERLAY_H
#define BATTLEOVERLAY_H

namespace BattleOverlay
{
    // Called when the actor's overlay is drawn.
    void OnOverlayDrawActor(RPG::Actor *actPtr);


    // Called when the monsters's overlay is drawn.
    void OnOverlayDrawMonster(RPG::Monster *monPtr);

    // The number of battlers drawn this frame and the numbr of valid entrys in the id-array.
    unsigned int battlersDrawn = 0;
    // Contains the id of battler that were drawn according to onBattlerDrawn callbacks  this frame.
    int drawnBattlerIds[8+4] = { 0,0,0,0,0,0,0,0,0,0,0,0};

    // Needs to be called directly on top of all actors and before the HUD and Menus are drawn.
    // Called in onDrawBattleActionWindow's first call, in which the 'selActive' parameter is false.
    void DrawOverlay()
    {
        for(unsigned int iBattler = 0; iBattler < battlersDrawn ; iBattler++) // Call OnOverlayDraw function according to battler type.
        {
            int thisId = drawnBattlerIds[iBattler];
            // Heros are taken by database ID. Monsters were saved as ( -1 to -8 ) and are accessed as ( 0 to 7 ).
            thisId > 0 ? OnOverlayDrawActor(RPG::actors[thisId]) : OnOverlayDrawMonster(RPG::monsters[(-thisId) -1]);
        }

        battlersDrawn = 0;
    }

    // When a battler is drawn, this function is called. The battlers ID is saved into an array.
    // Called in onBattlerDrawn for both heros and monsters.
    void RememberBattlerDrawn ( RPG::Battler *battler )
    {
        // For Heroes, this is the Database-ID ( 1-based ), for Monsters it's the 1-based monsterGroup-ID. Both are > 0.
        int id = battler->id;
        // Save the ID into the array with monster-IDs saved as negative values.
        drawnBattlerIds[battlersDrawn] = battler->isMonster() ?  -id : id;
        battlersDrawn++;
    }
}

#endif // BATTLEOVERLAY_H
